const generators = require('yeoman-generator');
const Storage =  require('yeoman-generator/lib/util/storage');

const colors = require('colors');
const git = require('simple-git');
const fs = require('fs');
const path = require('path');
const rimraf = require('rimraf');
const extend = require('../../lib/extend.js');

const logoProject = require('../../lib/logo.js');

const Imprimatur = function (yo, callback) {

  this.yo = yo;

  this.initPackageConfig();

  this.initLocalConfig();

  this.copyFiles();

  callback.call(this);

};

Imprimatur.prototype.initPackageConfig = function () {
  if (!fs.existsSync(this.yo.destinationPath('package.json'))) {
    this.yo.template(
      this.yo.templatePath('package.json'),
      this.yo.destinationPath('package.json'),
      {
        name : "imprimatur-project",
        description : "Project on Imprimatur CMS"
      }
    );
  }
};

Imprimatur.prototype.initLocalConfig = function () {
  if (!fs.existsSync(this.yo.destinationPath('local.json'))) {
    this.yo.fs.copy(
      this.yo.templatePath('imp','local.default.json'),
      this.yo.destinationPath('local.json')
    );
  }
};

Imprimatur.prototype.copyFiles = function () {

  this.yo.fs.copy(
    this.yo.templatePath('imp','data'),
    this.yo.destinationPath('data')
  );

  this.yo.fs.copy(
    this.yo.templatePath('imp','exec'),
    this.yo.destinationPath('exec')
  );

  this.yo.fs.copy(
    this.yo.templatePath('imp','www'),
    this.yo.destinationPath('www')
  );

  this.yo.fs.copy(
    this.yo.templatePath('imp','auto.p'),
    this.yo.destinationPath('auto.p')
  );

  console.log('Copy files');

};

module.exports = generators.Base.extend({
  constructor: function () {
    generators.Base.apply(this, arguments);

    this.argument('sourcePathArgument', { type: String, required: true });
    this.sourceRoot(this.sourcePathArgument);

    this.argument('destinationPathArgument', { type: String, required: true });
    this.destinationRoot(this.destinationPathArgument);

    this.argument('isTest', { type: String, required: false });


    // Метод для получения ответов генератора
    this.getAnswers = function (value) {
      if (value && this.answers && this.answers[value]) {
        return this.answers[value];
      }
      return undefined;
    }.bind(this);
  },

  prompting: function () {
    if (!this.isTest) {
      logoProject.print();
    }

    this.answers = new Storage('app', this.fs, this.destinationPath('local.json')).getAll();

    var prompts = [{
        type: 'input',
        name: 'path',
        message: 'Директория для установки проекта:',
        when: function () {
          return !this.answers['path'];
        }.bind(this),
        default: this.destinationPath(),
        filter: function (value) {
          return path.resolve(this.destinationPath(), value);
        }.bind(this)
    }, {
        type    : 'confirm',
        name    : 'confirm',
        when: function () {
          return !this.answers['confirm'];
        }.bind(this),
        message : function (answers) {
          return 'Установить в директорию ' + colors.yellow(answers.path) + '?';
        }
    }];

    return this.prompt(prompts).then(function (answers) {
      // To access props later use this.props.someAnswer;
      extend(this.answers, answers);
    }.bind(this));
  },

  writing: function () {
    var done = this.async();

    if (!this.answers.confirm) {
      return;
    }

    if( fs.existsSync(this.templatePath('imp'))) {
      rimraf.sync(this.templatePath('imp'));
    }

    // Create dir from Imprimatur
    fs.mkdirSync(this.templatePath('imp'));

    // ------------------------------
    // Read documentation simple-git
    // https://www.npmjs.com/package/simple-git
    git(this.templatePath('imp'))
      .init()
      .addRemote('imp', 'https://gitlab-ci-token:sp5zx3Ws8mxFBN3SS1yi@gitlab.design.ru/imprimatur/imp1.git')
      .pull('imp', 'test-2')
      .then(function () {

        this.imprimatur = new Imprimatur(this, function () {
          this.localConfig = new Storage('app', this.fs, this.destinationPath('local.json'));
          this.localConfig.set(this.answers);
          done();
          console.log('Repo cloning');
        }.bind(this));

      }.bind(this));
  },

  // install: function () {
    // this.runInstall('npm', '', {'prefix': './'});
    // this.spawnCommandSync('yarn');
  // },

  end: function () {
    if( fs.existsSync(this.templatePath('imp'))) {
      rimraf.sync(this.templatePath('imp'));
    }
  }

});
