const generators = require('yeoman-generator');
const Storage =  require('yeoman-generator/lib/util/storage');
const colors = require('colors');
const extend = require('../../lib/extend.js');

const logoProject = require('../../lib/logo.js');

module.exports = generators.Base.extend({

  constructor: function () {
    generators.Base.apply(this, arguments);

    this.argument('sourcePathArgument', { type: String, required: true });
    this.sourceRoot(this.sourcePathArgument);

    this.argument('destinationPathArgument', { type: String, required: true });
    this.destinationRoot(this.destinationPathArgument);

    // Метод для получения ответов генератора
    this.getAnswers = function (value) {
      if (value && this.answers && this.answers[value]) {
        return this.answers[value];
      }
      return undefined;
    }.bind(this);

    this.localConfig = new Storage('database', this.fs, this.destinationPath('local.json'));
  },

  prompting: function () {

    logoProject.mysql();

    this.answers = this.localConfig.getAll();

    var prompts = [{
      type: 'list',
      name: 'mode',
      message: 'Настроить подключение к базе данных?',
      when: function () {
        return !this.answers['mode'];
      }.bind(this),
      choices: [
        {name: 'Задать строку подключения', value: 'string'},
        // {name: 'Указать параметры', value: 'master'},
        {name: 'Настроить потом в local.json', value : 'skip'}
      ]
    }, {
      type: 'input',
      name: 'connect-string',
      message: 'Строка подключения:',
      when: function (answers) {
        return answers['mode'] === 'string' && !this.answers['mode'];
      }.bind(this),
      default: this.answers['connect-string']
    }];

    return this.prompt(prompts).then(function (answers) {
      extend(this.answers, answers);
    }.bind(this));
  },

  config: function () {
    var localConfig = new Storage('database', this.fs, this.templatePath('local.json')).getAll();
    extend(localConfig, this.answers);
    this.localConfig.set(localConfig);
  }

});
