const generators = require('yeoman-generator');
const Storage =  require('yeoman-generator/lib/util/storage');
const colors = require('colors');
const extend = require('../../lib/extend.js');

const logoProject = require('../../lib/logo.js');

module.exports = generators.Base.extend({

  constructor: function () {

    generators.Base.apply(this, arguments);

    this.argument('sourcePathArgument', { type: String, required: true });
    this.sourceRoot(this.sourcePathArgument);

    this.argument('destinationPathArgument', { type: String, required: true });
    this.destinationRoot(this.destinationPathArgument);

    this.localParserConfig = new Storage('parser', this.fs, this.destinationPath('local.json'));

    // Метод для получения ответов генератора
    this.getAnswers = function (value) {
      if (value && this.answers && this.answers[value]) {
        return this.answers[value];
      }
      return undefined;
    }.bind(this);
  },

  prompting: function () {

    logoProject.parser();

    this.answers = this.localParserConfig.getAll();

    var prompts = [{
      type: 'list',
      name: 'os',
      message: 'Какую версию Парсера установить в папку проекта?',
      when: function (){
        return !this.answers['os'];
      }.bind(this),
      choices: [
        {name: 'Windows', value: 'windows'},
        {name: 'Mac OS', value : 'osx'},
        {name: 'Linux', value : 'linux'},
        {name: 'Никакой', value : 'none'}
      ]
    }];

    return this.prompt(prompts).then(function (answers) {
      extend(this.answers, answers);
    }.bind(this));
  },

  config: function () {
    var localConfig = new Storage('parser', this.fs, this.templatePath(this.answers.os, 'local.json')).getAll();
    extend(localConfig, this.answers);
    this.localParserConfig.set(localConfig);
  },

  writing: function () {
    if (!this.answers.os.match(/^(windows|osx|linux|none)$/)) {
      return false;
    }

    this.directory(
      this.templatePath(this.answers.os, 'cgi-bin'),
      this.destinationPath('cgi-bin')
    );
  }

});
