const generators = require('yeoman-generator');
const Storage =  require('yeoman-generator/lib/util/storage');
const colors = require('colors');
const path = require('path');
const fs = require('fs');
const extend = require('../../lib/extend.js');

const logoProject = require('../../lib/logo.js');

module.exports = generators.Base.extend({

  constructor: function () {

    generators.Base.apply(this, arguments);

    this.argument('sourcePathArgument', { type: String, required: true });
    this.sourceRoot(this.sourcePathArgument);

    this.argument('destinationPathArgument', { type: String, required: true });
    this.destinationRoot(this.destinationPathArgument);

    // Метод для получения ответов генератора
    this.getAnswers = function (value) {
      if (value && this.answers && this.answers[value]) {
        return this.answers[value];
      }
      return undefined;
    }.bind(this);

    this.localServerConfig = new Storage('server', this.fs, this.destinationPath('local.json'));
  },

  prompting: function () {

    logoProject.nodejs();

    this.answers = this.localServerConfig.getAll();

    var prompts = [{
      type: 'list',
      name: 'mode',
      message: 'Установить сервер?',
      when: function () {
        return !this.answers['mode'];
      }.bind(this),
      choices: [
        {name: 'Да', value: true},
        {name: 'Нет', value : false}
      ]
    }];

    return this.prompt(prompts).then(function (answers) {
      extend(this.answers, answers);
    }.bind(this));
  },

  config: function () {
    var localConfig = new Storage('server', this.fs, this.templatePath('local.json')).getAll();
    extend(localConfig, this.answers);
    this.localServerConfig.set(localConfig);

    const scripts = new Storage('scripts', this.fs, this.destinationPath('package.json'));
    scripts.set('server', 'node server/app');
  },

  writing: function () {
    this.directory(
      this.templatePath('server'),
      this.destinationPath('server')
    );
  }

});
