var http = require('http');
var express = require('express');
var path = require('path');
var favicon = require('serve-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var helmet = require('helmet');
var gateway = require(path.join(__dirname, 'lib/gateway'));
var pkg = require('../local.json');

var app = express();

app.use(helmet());
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({ extended: false }));
app.use(cookieParser());

app.use(gateway(path.join(__dirname, pkg.server.public_path), {
    '.html': path.join(__dirname, '../', pkg.parser.path),
    imprimatur: true
}));

app.use(express.static(path.join(__dirname, pkg.server.public_path)));

// catch 404 and forward to error handler
app.use(function(req, res, next) {
    var err = new Error('Not Found');
    err.status = 404;
    next(err);
});

// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
    app.use(function(err, req, res, next) {
        res.status(err.status || 500);
        res.status(500).json({
            message: err.message,
            error: err
        });
    });
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
    res.status(err.status || 500);
    res.status(500).json({
        message: err.message,
        error: err
    });
});

var server = http.createServer(app);

server.listen(pkg.server.port, pkg.server.host);
