/**
 * ImpIT
 * Imprimatur Installation Tool
 */

const yeoman = require('yeoman-environment').createEnv();
const process = require('process');

const colors = require('colors');
const pathProject = require('./lib/path.js');
const logoProject = require('./lib/logo.js');

var Imprimatur = function () {

  require('./lib/access.js')
    .test()
    .then(function () {
      if (!process.env.npm_config_global) {
        this.installCMS()
          .then(
            this.installParser.bind(this)
          ).then(
          this.installServer.bind(this)
        ).then(
          this.configDatabase.bind(this)
        ).then(
          this.configDeveloper.bind(this)
        ).then(
          this.install.bind(this)
        ).then(function () {
          logoProject.cancel();
        }).catch(function (code) {
          console.log('CODE ERROR', code);
        });
      }
    }.bind(this))
    .catch(function (code) {
      switch(code){
        case 'access':
          console.log(colors.red('\r\t---------------------------------------------------------------------' +
            '\n\n\tУ вас нет прав на установку данного программного обеспечения' +
            '\n\n\tЕсли вы не являетесь сотрудником студии, то можете попробовать им стать' +
            '\n\tРаздел вакансий http://www.artlebedev.ru/job/' +
            '\n\n\tСтудия Артемия Лебедева' +
            '\n\thttp://www.artlebedev.ru/' +
            '\n\tmailbox@artlebedev.ru\n' +
            '\n\t---------------------------------------------------------------------\n\n'
          ));
          break;
        default:break;
      }
    });
};

Imprimatur.prototype.installCMS = function () {
  yeoman.registerStub(require(pathProject.getSourcePath('generators/app/index.js')), 'npm:imprimatur');

  return new Promise(function (resolve, reject) {
    yeoman.run('npm:imprimatur ' + pathProject.getSourcePath('generators/app/templates') + ' ' + pathProject.getDestinationPath(), function () {
      resolve();
    }).on('end', function () {
      // this - generator context
      // this.getAnswers - function in generators returned answers by name
      pathProject.setDestinationPath(this.getAnswers('path'));

      if (!this.getAnswers('confirm')) {
        reject('cancel');
      }
    });
  });
};

Imprimatur.prototype.installParser = function () {
  yeoman.registerStub(require(pathProject.getSourcePath('generators/parser/index.js')), 'npm:imprimatur-parser');

  return new Promise(function (resolve) {
    yeoman.run('npm:imprimatur-parser ' + pathProject.getSourcePath('generators/parser') + ' ' + pathProject.getDestinationPath(), function () {
      resolve();
    });
  });
};

Imprimatur.prototype.installServer = function () {
  yeoman.registerStub(require(pathProject.getSourcePath('generators/server/index.js')), 'npm:imprimatur-server');

  return new Promise(function (resolve) {
    yeoman.run('npm:imprimatur-server ' + pathProject.getSourcePath('generators/server') + ' ' + pathProject.getDestinationPath(), function () {
      resolve();
    });
  });
};

Imprimatur.prototype.configDatabase = function () {
  yeoman.registerStub(require(pathProject.getSourcePath('./generators/database/index.js')), 'npm:imprimatur-database');

  return new Promise(function (resolve) {
    yeoman.run('npm:imprimatur-database ' + pathProject.getSourcePath('generators/database') + ' ' + pathProject.getDestinationPath(), function () {
      resolve();
    });
  });
};

Imprimatur.prototype.configDeveloper = function () {
  yeoman.registerStub(require(pathProject.getSourcePath('generators/developer/index.js')), 'npm:imprimatur-developer');

  return new Promise(function (resolve) {
    yeoman.run('npm:imprimatur-developer ' + pathProject.getSourcePath('generators/developer') + ' ' + pathProject.getDestinationPath(), function () {
      resolve();
    });
  });
};

Imprimatur.prototype.install = function () {
  yeoman.registerStub(require(pathProject.getSourcePath('generators/app/install.js')), 'npm:imprimatur-install');

  return new Promise(function (resolve) {
    yeoman.run('npm:imprimatur-install ' + pathProject.getSourcePath('generators/app') + ' ' + pathProject.getDestinationPath(), function () {
      resolve();
    });
  });
};

module.exports = new Imprimatur();
